Imports Microsoft.VisualBasic
Imports System
Imports developwithpassion.bdd
Imports developwithpassion.bdd.contexts
Imports developwithpassion.bdd.mbunit.standard.observations
Imports Rhino.Mocks

Namespace Domain.Specs
    Public Class RegisterSpecs
        Public Class concern
            Inherits observations_for_a_sut_without_a_contract(Of Register)
            Private c As context = AddressOf context
            Private Sub context()
                receiptReceiver = the_dependency(Of IReceiptReceiver)()
                product_catalog = the_dependency(Of IProductCatalog)()
            End Sub
            Protected Shared receiptReceiver As IReceiptReceiver
            Protected Shared product_catalog As IProductCatalog
        End Class

        Public Class when_completing_a_sale_that_was_started_but_has_no_items_added
            Inherits concern
            Private a As after_the_sut_has_been_created = AddressOf after_the_sut_has_been_created_do
            Private Sub after_the_sut_has_been_created_do()
                sut.NewSaleInitiated()
            End Sub
            Private b As because = AddressOf because
            Private Sub because()
                sut.SaleCompleted()
            End Sub
            Private should_calculate_a_sales_receipt_of_zero As it = Function() receiptReceiver.received(AddressOf should_calculate_a_sales_receipt_of_zero_do)
            Private Sub should_calculate_a_sales_receipt_of_zero_do(ByVal x As IReceiptReceiver)
                x.ReceiveTotalDue(zero_dollars)
            End Sub
            Private Shared ReadOnly zero_dollars As New Money(0D)
        End Class

        Public Class when_attempting_to_complete_a_sale_that_was_not_started
            Inherits concern
            Private b As because = AddressOf because
            Private Sub because()
                sut.SaleCompleted()
            End Sub

            Private should_not_create_a_receipt As it = AddressOf it_should_not_create_a_receipt
            Private Sub it_should_not_create_a_receipt()
                receiptReceiver.never_received(AddressOf receiptReceiverNeverReceived)
            End Sub
            Private Sub receiptReceiverNeverReceived(ByVal x As IReceiptReceiver)
                x.ReceiveTotalDue(Arg(Of Money).[Is].Anything)
            End Sub

        End Class

        Public Class when_completing_a_sale_that_has_multiple_items_added_to_it
            Inherits concern
            Private c As context = AddressOf context
            Private Sub context()
                item_id_1 = New ItemId("000000001")
                item_id_2 = New ItemId("000000002")
                description_for_item_with_id1 = New ProductDescription("description 1", when_completing_a_sale_that_has_multiple_items_added_to_it.price_of_item_1)
                price_of_item_1 = New Money(3D)
                price_of_item_2 = New Money(7D)
                description_for_item_with_id2 = New ProductDescription("description 2", when_completing_a_sale_that_has_multiple_items_added_to_it.price_of_item_2)
                single_item = New Quantity(1)

                product_catalog.Stub(Function(x) x.ProductDescriptionFor(item_id_1)).Return(description_for_item_with_id1)
                product_catalog.Stub(Function(x) x.ProductDescriptionFor(item_id_2)).Return(description_for_item_with_id2)
            End Sub
            Private a As after_the_sut_has_been_created = AddressOf after_the_sut_has_been_created
            Private Sub after_the_sut_has_been_created()
                sut.NewSaleInitiated()
                sut.ItemEntered(item_id_1, single_item)
                sut.ItemEntered(item_id_2, single_item)
            End Sub
            Private b As because = AddressOf because
            Private Sub because()
                sut.SaleCompleted()
            End Sub
            Private should_prepare_a_sales_receipt_with_the_total_cost_of_the_items_added As it = Function() receiptReceiver.received(AddressOf receiptReceiverReceived)
            Private Sub receiptReceiverReceived(ByVal x As IReceiptReceiver)
                x.ReceiveTotalDue(price_of_item_1 + price_of_item_2)
            End Sub


            Private Shared item_id_1 As ItemId
            Private Shared item_id_2 As ItemId
            Private Shared description_for_item_with_id1 As ProductDescription
            Private Shared description_for_item_with_id2 As ProductDescription
            Private Shared single_item As Quantity
            Private Shared price_of_item_1 As Money
            Private Shared price_of_item_2 As Money
        End Class
    End Class
End Namespace