Imports Microsoft.VisualBasic
Imports System
Imports NMock2
Imports NUnit.Framework

Namespace Domain.Tests
	<TestFixture> _
	Public Class CommandParserTests
		Private mockery As Mockery
		Private commandParser As CommandParser
		Private saleEventListener As ISaleEventListener

		<SetUp> _
		Public Sub BeforeTest()
			mockery = New Mockery()
			saleEventListener = mockery.NewMock(Of ISaleEventListener)()
			commandParser = New CommandParser(saleEventListener)
		End Sub

		<TearDown> _
		Public Sub AfterTest()
			mockery.VerifyAllExpectationsHaveBeenMet()

		End Sub

		<Test> _
		Public Sub NotifiesListenerOfNewSaleEvent()
			Const newSaleCommand As String = "Command:NewSale"

			Expect.Once.On(saleEventListener).Method("NewSaleInitiated")

			commandParser.Parse(newSaleCommand)
		End Sub


		<Test> _
		Public Sub NotifiesListenerOfSaleCompletedEvent()
			Const endSaleCommand As String = "Command:EndSale"

			Expect.Once.On(saleEventListener).Method("SaleCompleted")

			commandParser.Parse(endSaleCommand)
		End Sub


		<Test> _
		Public Sub NotifiesListenerOfItemAndQuantityEntered()
			Const commandWithBarcodeAndSingleQuantity As String = "Input: Barcode=100008888559, Quantity=1"
			Dim expectedItemId = New ItemId("100008888559")
			Dim singleQuantity = New Quantity(1)

			Expect.Once.On(saleEventListener).Method("ItemEntered").With(expectedItemId, singleQuantity)

			commandParser.Parse(commandWithBarcodeAndSingleQuantity)
		End Sub

	End Class
End Namespace