Imports Microsoft.VisualBasic
Imports System
Imports NMock2
Imports NUnit.Framework

Namespace Domain.Tests
	<TestFixture> _
	Public Class RegisterTests
		Private mockery As Mockery
		Private receiptReceiver As IReceiptReceiver
		Private register As Register
		Private ReadOnly itemId1 As New ItemId("000000001")
		Private ReadOnly itemId2 As New ItemId("000000002")
		Private Shared ReadOnly priceOfItem2 As New Money(7.00D)
		Private Shared ReadOnly priceOfItem1 As New Money(3.00D)

		Private ReadOnly descriptionForItemWithId1 As New ProductDescription("description 1", priceOfItem1)

		Private ReadOnly descriptionForItemWithId2 As New ProductDescription("description 2", priceOfItem2)
		Private ReadOnly singleItem As New Quantity(1)
		Private productCatalog As IProductCatalog


		<SetUp> _
		Public Sub BeforeTest()
			mockery = New Mockery()
			receiptReceiver = mockery.NewMock(Of IReceiptReceiver)()
			productCatalog = mockery.NewMock(Of IProductCatalog)()
			register = New Register(receiptReceiver, productCatalog)

            Stub.On(productCatalog).Method("ProductDescriptionFor").With(itemId1).Will([Return].Value(descriptionForItemWithId1))

            Stub.On(productCatalog).Method("ProductDescriptionFor").With(itemId2).Will([Return].Value(descriptionForItemWithId2))

		End Sub

		<TearDown> _
		Public Sub AfterTest()
			mockery.VerifyAllExpectationsHaveBeenMet()
		End Sub

		<Test> _
		Public Sub ReceiptTotalForASaleWithNoItemsShouldBeZero()

			register.NewSaleInitiated()

			Dim zeroDollars = New Money(0D)
			Expect.Once.On(receiptReceiver).Method("ReceiveTotalDue").With(zeroDollars)

			register.SaleCompleted()
		End Sub


		<Test> _
		Public Sub ShouldNotSendRecieptWhenThereIsNoSale()
			Expect.Never.On(receiptReceiver)
			register.SaleCompleted()
		End Sub

		<Test> _
		Public Sub ShouldCalculateRecieptForSaleWithMultipleItemsOfSingleQuantity()
			register.NewSaleInitiated()

			register.ItemEntered(itemId1, singleItem)

			register.ItemEntered(itemId2, singleItem)


			Expect.Once.On(receiptReceiver).Method("ReceiveTotalDue").With(priceOfItem1 + priceOfItem2)


			register.SaleCompleted()
		End Sub


	End Class
End Namespace